# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#!/usr/local/bin/perl -s
#
# calc_cooccurrence.perl: Calculate co-occurrence statistics for the input,
# each line of which represents a 2x2 contingency table.
#
# example input; [Dunning 93]
#
#      # k(AB) k(A~B) k(~AB) k(~A~B)  w1  w2
#          110   2442    111   29114  the swiss
#           29     13    123   31612  can be
#
# This is a streamlined version of calc_multi_x2.perl that just displays
# the data formatted as a table.
#
# sample output:
#
# joint   dice    jaccard MI      x2       g2      avg-MI
# 0.003   0.079   0.041   2.632   525.021  270.722 0.006   the     swiss
# 0.001   0.299   0.176   7.173   4153.695 263.897 0.006   can     be
#
# TODO:
# - add options for disabling particular metrics
#

# Load in the common module, making sure the script dir is in Perl's lib path
$dir = `dirname $0`; chop $dir; unshift(@INC, $dir);
require 'common.perl';

# Load in other modules
require 'cooccurrence.perl';

# Check command-linme arguments
if (!defined($ARGV[0])) {
    my($options) = "options = [-standard | -totals] [ data | - ]";
    my($example) = "examples:\n\n";
    $example .=  "echo 110 2442 111 29114 the swiss | $0 -\n\n";
    my($note) = "";
    ## $note .= "notes:\n\nSome usage note.\n";			# TODO: add optional note

    die "\nusage: $0 [options]\n\n$options\n\n$example\n$note\n";
}
&init_var(*totals, &FALSE);		# input gives totals: A B AB N
&init_var(*standard, ! $totals);	# standard input: AB A_~B B_~A ~A_~B

# Display the table header
print "# ", join("\t", get_cooccurrence_metrics()), "\n";

# If data given on the command line, process it and exit
if (defined($ARGV[0]) && ($ARGV[0] =~ /^\d+$/)) {
    &calc_coccur_helper(" @ARGV ");
    exit;
}

# Process each line of input as a separate 2x2 contingency table
#
while (<>) {
    chomp;
    &dump_line($_);

    # Skip comments
    s/#.*//g;
    if (/^ *$/) {
	next;
    }

    # Extract counts and calculate statistics
    &calc_cooccur_helper($_);
}

#------------------------------------------------------------------------------

# calc_coccur_helper(line_of_data)
# 
# Read each of k(AB) k(A~B) k(~AB) and k(~A~B); also read the bigram tokens
# Note: the Perl variables use '_' instead of ~
#      \d: matches numeric         \D: matches non-numeric
#      \w: matches alphanumeric    \W: matches non-alphanumeric
#
sub calc_cooccur_helper {
    &debug_print(&TL_DETAILED, "calc_cooccur_helper(@_)\n");
    my($data) = @_;

    if ($data =~ /(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/) {
	my($AB, $A_B, $_AB, $_A_B) = ($1, $2, $3, $4);
	my($rest) = $';
	if (! $standard) {
	    $AB = $1;
	    my($A) = $2;
	    my($B) = $3;
	    my($N) = $4;

	    $A_B = $A - $AB;
	    $_AB = $B - $AB;
	    $_A_B = $N - ($AB + $A_B + $_AB);
	}

	# Check for the bigram word or phrases
	my($bigram1) = "";
	my($bigram2) = "";
	($bigram1, $rest) = &parse_token($rest);
	($bigram2, $rest) = &parse_token($rest);

	# Calculate the co-occurrence statistics
	my(@metrics) = &calc_cooccurrence($AB, $A_B, $_AB, $_A_B, 
					  $bigram1, $bigram2);
	print join("\t", &round_all(@metrics));
	print "\t$bigram1\t$bigram2\n";
    }
    else {
	&warning_out("Ignoring data '$_'\n");
    }
}

#------------------------------------------------------------------------

# parse_token(text): Parse a token that optionally is quoted.
# The token is returned along with the remainder of the text.
#
# ex: parse_token(" '(SonOf Sam)' Julie ") => ("(SonOf Sam)", " Julie")
#
sub parse_token {
    my($text) = @_;
    my($token) = "";
    my($rest) = "";

    # First check for quoted string
    if ($text =~ /^\s*([\'\"])([^\1]*)\1/) {
	$token = $2;
	$rest = $';
    }

    # Otherwise check for nonblank token
    elsif ($text =~ /^\s*(\S+)/) {
	$token = $1;
	$rest = $';
    }
    &debug_print(&TL_VERY_VERBOSE, "parse_token(@_) => ($token, $rest)\n");

    return ($token, $rest);
}
