# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#!/usr/bin/perl -sw
#
# extra.perl: common module with extraneous but occasionally useful functions
#

# Load in the common module, making sure the script dir is in the Perl lib path
BEGIN { my $dir = `dirname $0`; chomp $dir; unshift(@INC, $dir); }
require 'common.perl';
## use HotKey;

$topN_options  = "[-TOPN=num]";
&init_var(*TOPN, 5);

# options for Web frequency checks
&init_var(*hotbot, &FALSE);
&init_var(*altavista, (! $hotbot));
&init_var(*use_MI, &FALSE);
&init_var(*use_dice, (! $use_MI));
&init_var(*null_eof, &FALSE);
&init_var(*numeric_format, "%.3f");	# numeric format for rounding and normalization

# Associative arrays for abbreviations
%months = ("january", "jan",
	   "february", "feb",
	   "march", "mar",
	   "april", "apr",
	   "may", "may",
	   "june", "jun",
	   "july", "jul",
	   "august", "aug",
	   # "september", "sept",
	   "september", "sep",
	   "october", "oct",
	   "november", "nov",
	   "december", "dec");
%weekdays = ("sunday", "sun",
	     "monday", "mon",
	     "tuesday", "tues",
	     "wednesday", "wed",
	     "thursday", "thurs",
	     "friday", "fri",
	     "saturday", "sat");

# Alternative format for converting months into indices
@month_names = ("\tjanuary\tjan\t",
	   "\tfebruary\tfeb\t",
	   "\tmarch\tmar\t",
	   "\tapril\tapr\t",
	   "\tmay\tmay\t",
	   "\tjune\tjun\t",
	   "\tjuly\tjul\t",
	   "\taugust\taug\t",
	   # "\tseptember\tsept\t",
	   "\tseptember\tsep\t",
	   "\toctober\toct\t",
	   "\tnovember\tnov\t",
	   "\tdecember\tdec\t");


#------------------------------------------------------------------------------

# routines for maintaining a list of the N best (key, value) pairs
#
# sample usage:
#
#     require 'extra.perl'
#     ...
#
#     my(@max_label, @max_value);
#     &init_topN(5);
#     &clear_topN(\@max_label, \@max_value);
#     ...
#     while (...) {
#         &revise_topN(\@max_label, \@max_value, $label, $value);
#     }
#     ...
#     &show_topN(\@max_label, \@max_value);
#

## $topN_options = "[-TOPN=num]";

# init_topN(num): initialize the topN settings 
# TODO: revise to clear the arrays
#
sub init_topN {
    local($num) = @_;
    
    if (defined($num)) {
	$TOPN = $num;
    }
    &debug_print(5, "init_topN(@_): TOPN=$TOPN\n");

    return;
}

# clear_topN(): reset the list of the topN entries
#
sub clear_topN {
    my($max_label_ref, $max_value_ref) = @_;
    &debug_print(5, "clear_topN(@_)\n");

    @$max_label_ref = ();
    @$max_value_ref = ();

    return;
}


# revise_topN(top_keys, best_values, key, value)
#
# Revise the list of the top N values and associated keys, given the
# current key and value.
#

sub revise_topN {
    local($max_label_ref, $max_value_ref, $label, $value) = @_;
    local($i);
    &debug_print(5, "revise_topN(@_)\n");
    &assert('$label ne ""');

    # Use as the best one if it is higher
    # Find the position in the list where it belongs
    local($pos) = -1;
    for ($i = 0; $i < $TOPN; $i++) {
	# NOTE: ties might eventually fall off the list
	if ((!defined($$max_value_ref[$i])) 
	    || ($value > $$max_value_ref[$i])) {
	    $pos = $i;
	    last;
	}
    }
    if ($pos == -1) {
	return;
    }

    # Shift the lesser entries down the list
    for ($i = ($TOPN - 1); $i > $pos; $i--) {
	if (defined($$max_value_ref[$i - 1])) {
	    $$max_value_ref[$i] = $$max_value_ref[$i - 1];
	    $$max_label_ref[$i] = $$max_label_ref[$i - 1];
	}
    }
    $$max_value_ref[$pos] = $value;
    $$max_label_ref[$pos] = $label;
    &trace_array($max_label_ref);
    &trace_array($max_value_ref);

    return;
}


# show_topN(top_keys, best_values, [FILE="STDOUT"], [display_format="%s\t%.3f"])
#
# Display the (sorted) list of the top N key/value pairs.
#
sub show_topN {
    my($max_label_ref, $max_value_ref, $FILE, $display_format) = @_;
    $FILE = "STDOUT" if (! defined($FILE));
    $display_format = "%s\t%.3f\n" unless (defined($display_format));
    &debug_print(5, "show_topN(@_)\n");
    my($i);

    for ($i = 0; $i < $TOPN; $i++) {
	printf $FILE $display_format, $$max_label_ref[$i], $$max_value_ref[$i] 
	    unless (!defined($$max_label_ref[$i]));
    }

    return;
}

#------------------------------------------------------------------------------

# abbreviate(text, abbreviations)
#
# Make abbreviations for all occurrences of the keys defined in the
# associative array.
#
sub abbreviate {
    local ($text, *abbrevs) = @_;
    local ($key);
    &debug_print(6, "abbreviate(@_)\n");
    &debug_out(99, "keys(abbrevs) = %s\n", join(' ', keys(%abbrevs)));

    # Substitute the abbreviation for each key in the table
    foreach $key (keys(%abbrevs)) {
	local($abbreviation) = &get_entry(*abbrevs, $key, "");
	&assert('$abbreviation ne ""');
	$text =~ s/$key/$abbreviation/g;
    }

    return ($text);
}


# abbreviate_dates(text)
#
# Abbreviate the date expressions in the text.
#
sub abbreviate_dates {
    local($text) = @_;

    $text = &abbreviate($text, *months);
    $text = &abbreviate($text, *weekdays);
    return ($text);
}

# month_value(month): Returns numeric value of Month (e.g., July is 7).
# In technical terms, this is the 1-based index.
#
sub month_value {
    local($month) = @_;
    local($value) = 0;
    local($i);

    for ($i = 0; $i <= $#month_names; $i++) {
	 if ($month_names[$i] =~ /\t$month\t/i) {
	     $value = $i + 1;
	 }
    }
    &debug_print(6, "month_value(@_) => $value\n");

    return ($value);
}

# month_abbrev(month): returns abbreviation of n-th Month (e.g., 7 => jul).
#
sub month_abbrev {
    local($value) = @_;
    local($month) = "???";

    if (($value > 0) && ($value <= ($#month_names + 1))) {
	$month = $month_names[$value - 1];
	$month =~ s/\t\S+\t(\S+)\t/$1/;
    }
    &debug_print(6, "month_abbrev(@_) => $month\n");

    return ($month);
}

# format_assoc_data_aux(sort_routine, assoc_data_ref, [max, freq_delim, entry_delim])
# Returns a string representation for a frequency list stored in an associative array.
# 
sub format_assoc_data_aux {
    my($sort_routine, $freq_assoc_ref, $max, $freq_delim, $entry_delim) = @_;

    $max = &MAXINT if (!defined($max));
    $freq_delim = "\t" if (!defined($freq_delim));
    $entry_delim = "\n" if (!defined($entry_delim));
    my(@formatted_data);

    my($count) = 0;
    my($key);
    foreach $key (&$sort_routine($freq_assoc_ref)) {
	push (@formatted_data, (sprintf "%s%s%s", $key, $freq_delim, $$freq_assoc_ref{$key}));
	last if ($count++ >= $max);
    }

    return (join($entry_delim, @formatted_data));
}


# format_assoc_data(assoc_data_ref, [max, freq_delim, entry_delim])
# Returns a string representation for a frequency list stored in an associative array.
# 
sub format_assoc_data {
    return (&format_assoc_data_aux(\&sorted_hash_keys_alphabetic, @_));
}


# format_freq_data(assoc_data_ref, [max, freq_delim, entry_delim])
# Returns a string representation for a frequency list stored in an associative array.
# 
sub format_freq_data {
    return (&format_assoc_data_aux(\&sorted_hash_keys_reverse_numeric, @_));
}


# format_numeric_assoc_data(assoc_data_ref, [max, number_delim, entry_delim])
# Returns a string representation for a number list stored in an associative array.
# 
sub format_numeric_assoc_data {
    return (&format_assoc_data_aux(\&sorted_hash_keys_numeric, @_));
}


# sorted_hash_keys_reverse_numeric(hash_ref): returns the keys of the hash table sorted
# by reverse numeric order of the values
#
# variants:
# sorted_hash_keys_numeric		keys sorted by numeric order of the values
# sorted_hash_keys_alphabetic		keys sorted by alphatic order
# sorted_hash_keys_numeric_keys		keys sorted by numeric order
#
sub sorted_hash_keys_reverse_numeric {
    # TODO: rename to sorted_hash_keys_reverse_numeric_value
    my($hash_ref) = @_;
    return sort { $$hash_ref{$b} <=> $$hash_ref{$a} } keys(%$hash_ref);
}

sub sorted_hash_keys_numeric {
    # TODO: rename to sorted_hash_keys_numeric_value
    my($hash_ref) = @_;
    return sort { $$hash_ref{$a} <=> $$hash_ref{$b} } keys(%$hash_ref);
}

sub sorted_hash_keys_alphabetic {
    # TODO: rename to sorted_hash_keys_alphabetic_keys
    my($hash_ref) = @_;
    return sort keys(%$hash_ref);
}

sub sorted_hash_keys_numeric_keys {
    my($hash_ref) = @_;
    return sort { $a <=> $b} keys(%$hash_ref);
}

# get_web_frequency(phrase, [-type=""]): returns the web frequency for the phrase
#
sub get_web_frequency {
    my($phrase, $type) = @_;
    $type = "" if (!defined($type));
    my($freq) = 0;

    my($options) = "-bare";
    $options .= " -$type" unless ($type eq "");
    $options .= " -altavista" if ($altavista);
    $phrase =~ s/ \-/ \\\-/g;
    my($result) = &run_command_fmt("web_freq.perl $options \"%s\"", $phrase);
    if ($result =~ /^(\d+)/) {
	$freq = $1;
    }

    return ($freq);
}

# web_cooccurrence(foreign, english)
#
# returns measure of co-occurrence for the two terms in web searches, using
# either mutual information or the Dice coefficient
#
#   mutual information:
#	log2(P(X=1,Y=1) / P(X=1) * P(Y=1))
#
#   the Dice coefficient:
#	Dice(X,Y) = 2 * p(X=1,Y=1)/(p(X=1) + p(Y=1))
#
sub web_cooccurrence {
    my($foreign_phrase, $english_phrase) = @_;

#      my($boolean_query) = "$foreign $english";
#      $boolean_query =~ s/ / AND /;
#      my($freq_X_Y) = &get_web_frequency($boolean_query, "boolean");
#      my($freq_X) = &get_web_frequency($foreign, "all");
#      my($freq_Y) = &get_web_frequency($english, "all");
#      my($MI) = log($freq_X_Y / ($freq_X * freq_Y)) / log(2);

    # Do individual boolean queries for all combinations of the pair
    my($foreign) = $foreign_phrase;
    $foreign =~ s/ \-/ \\\-/g;
    $foreign =~ s/s+/ AND /g;
    my($english) = $english_phrase;
    $english =~ s/ \-/ \\\-/g;
    $english =~ s/\s+/ AND /g;
    my($freq_X1_Y1) = &get_web_frequency("($foreign) AND ($english)", "boolean");
    my($freq_X1_Y0) = &get_web_frequency("($foreign) AND (NOT ($english))", "boolean");
    my($freq_X0_Y1) = &get_web_frequency("(NOT ($foreign)) AND ($english)", "boolean");
    my($freq_X0_Y0) = &get_web_frequency("(NOT ($foreign)) AND (NOT ($english))", "boolean");

    # Calculate the mutual information metric
    my($total_freq) = $freq_X1_Y1 + $freq_X1_Y0 + $freq_X0_Y1 + $freq_X0_Y0;
    $total_freq = 1 if ($total_freq == 0);
    my($prob_X) = (($freq_X1_Y1 + $freq_X1_Y0) / $total_freq);
    my($prob_Y) = (($freq_X1_Y1 + $freq_X0_Y1) / $total_freq);
    my($prob_XY) = ($freq_X1_Y1 / $total_freq);
    my($metric) = 0;

    # Determine the co-occurrence metric
    if ($use_MI) {
	my($MI) = 0;
	if (($prob_X > 0) && ($prob_Y > 0) && ($prob_XY > 0)) {
	    $MI = log($prob_XY / ($prob_X * $prob_Y)) / log(2);
	}
	$metric = $MI;
    }
    if ($use_dice) {
	my($dice) = 0;
	if (($prob_Y > 0) && ($prob_XY > 0)) {
	    $dice = 2 * $prob_XY / ($prob_X + $prob_Y);
	}
	$metric = $dice;
    }

    # Trace contingency table in format suitable for calc_multi_x2.perl
    &debug_out(4, "%d %d %d %d %s %s\n", 
	       $freq_X1_Y1, $freq_X1_Y0, $freq_X0_Y1, $freq_X0_Y0,
	       $foreign_phrase, $english_phrase);

    return ($metric);
}

# read_frequencies(file, freq_list_ref, [freq_first=0])
# 
# Read in the frequencies from the frequency file, which has the key in 
# the first column followed by the count in the second.
#     <key><TAB><count>
# Blank lines are those starting with comments (either '#' or ';') are ignored.
#
# The result is stored in the associative array given in the second argument 
# (reference parameter), the total frequency is the return value.
# note: If a key occurs more than once, the sum is used.
#
sub read_frequencies {
    my($file, $freq_list_ref, $freq_first) = @_;
    $freq_first = &FALSE if (! defined($freq_first));
    &debug_print(&TL_DETAILED, "read_frequencies(@_)\n");

    if (! open(FREQ, "<$file")) {
        &error_out("unable to open frequency file: $file ($!)\n");
	return (0);
    }

    my($total_freq) = 0;
    while (<FREQ>) {
        &dump_line("freq: $_", &TL_MOST_VERBOSE);
        chop;
        s/^[;\#].*//;			# strip comments
        next if (/^\s*$/);		# ignore blank lines
        my($key, $count) = split('\t', $_);
	if ($freq_first) {
	    ($count, $key) = split('\t', $_);
	}
        next if (!defined($key));
	${$freq_list_ref}{$key} = 0 if (!defined(${$freq_list_ref}{$key}));
        ${$freq_list_ref}{$key} += $count;
        $total_freq += $count;
    }
    close(FREQ);
    &debug_print(&TL_DETAILED, "total frequency is $total_freq\n");

    return ($total_freq);
}

# hash_key(key, num_buckets): hash function based on internal Perl hashing scheme
#
# from hv.h in Perl source distribution (version 5.6.1-2):
#
#  #define PERL_HASH(hash,str,len) \
#       STMT_START { \
#          register const char *s_PeRlHaSh = str; \
#          register I32 i_PeRlHaSh = len; \
#          register U32 hash_PeRlHaSh = 0; \
#          while (i_PeRlHaSh--) \
#              hash_PeRlHaSh = hash_PeRlHaSh * 33 + *s_PeRlHaSh++; \
#          (hash) = hash_PeRlHaSh + (hash_PeRlHaSh>>5); \
#      } STMT_END
#
# note: this produces too many conflicts perhaps due to faulty overflow handling
#  
sub hash_key {
    my($key, $num_buckets) = @_;
    my($result) = 0;
    &debug_print(&TL_VERY_VERBOSE, "hash_key(@_)\n");

    # Require that all operations (in this block) be done using integer arithmetic
    use integer;

    my ($char);
    foreach $char (split(//, $key)) {
	&debug_out(&TL_VERY_VERBOSE, "result=$result ord(char)=%d\n", ord($char));
	$result = (($result * 33) + ord($char));
	$result += &MAXINT if ($result < 0);
	## $result = -$result if ($result < 0);
    }
    $result += ($result / 32);
    $result += &MAXINT if ($result < 0);
    ## $result = -$result if ($result < 0);
    &debug_print(&TL_VERY_VERBOSE, "result=$result\n");

    return ($result % $num_buckets);
}

# get_user_response(prompt, [wait=1]): Display prompt to the user and return single-key response
# (echoing result to terminal).
# NOTE: This function is incompatible with <STDIN> style input (see sysread in manual).
# Use readline() instead for line input.
# TODO: have a separate function for prompt-less input (see get_next_char)
#
local($get_user_response_init)=&FALSE;
#
sub get_user_response {
    my($prompt, $wait) = @_;
    $wait = &TRUE if (!defined($wait));
    &debug_print(&TL_VERY_VERBOSE, "get_user_response(@_)\n");

    # Initialization so that Hotkey module only loaded if needed
    # note: this avoid problem with background processing of the scripts
    if (! $get_user_response_initialized) {
	&debug_print(&TL_VERBOSE, "using Hotkey\n");
	eval "use HotKey;";
	$get_user_response_init = &TRUE;
    }

    # Set stardard output, and error to be unbuffered
    # TODO: just do this once
    select(STDERR); $| = 1;
    select(STDOUT); $| = 1;

    # Display the prompt and get single-character response
    print $prompt;
    my($response) = "";
    my($done);
    do {
	$response = readkey();
	$done = &TRUE;

	# If terminal EOF is being ignored, reset the completion flag on null input
	# NOTE: This circumvents problems with readkey() on older Linux configurations
	&debug_out(&TL_MOST_VERBOSE, "readkey() => %s (%d)\n", $response, ord($response));
	if ($null_eof && (ord($response) == 0)) {
	    $done = (! $wait);
	}
    } while (! $done);
    print "$response\n";

    # Issue error and quit if end of file was encountered
    if (ord($response) == 0) {
	&exit("Unexpected end of file\n");
    }

    return ($response);
}

# get_next_char(): Returns next character from input or null if not ready
#
local($get_next_char_init)=&FALSE;
#
sub get_next_char {
    # Initialization so that ReadKey module only loaded if needed
    # note: this avoid problem with background processing of the scripts
    if (! $get_next_char_init) {
	&debug_print(&TL_VERBOSE, "using Term::ReadKey\n");
	eval "use Term::ReadKey;";
	$get_next_char_init = &TRUE;
    }

    # Read the character, defaulting to ""
    &ReadMode('cbreak');
    my($char) = &ReadKey(-1);
    $char = "" if (! defined($char));
    &ReadMode('restore');

    return ($char);
}

# get_boolean_response(prompt): Display yes/no type prompt and return TRUE is Y is
# pressed, requiring the user to enter only Y or N (case insensitive). 
# NOTE: The output is echoed.
#
sub get_boolean_response {
    my($prompt) = @_;
    my($OK) = &FALSE;
    &debug_print(&TL_VERY_VERBOSE, "get_boolean_response(@_)\n");

    for (;;) {
	my($response) = &get_user_response($prompt);
	if ($response =~ /[yn]/i) {
	    $OK = ($response =~ /y/i);
	    last;
	}
	print "\b";
    }

    return ($OK);
}

# get_numeric_response(prompt, min-value, max-value): Display prompt for numeric response
# and return value, requiring the user to enter a value in the specified range. 
# NOTE: The output is echoed.
#
sub get_numeric_response {
    my($prompt, $min, $max) = @_;
    my($choice) = ($min - 1);
    &debug_print(&TL_VERY_VERBOSE, "get_numeric_response(@_)\n");

    for (;;) {
	my($response) = &get_user_response("$prompt ($min-$max): ");
	if ($response =~ /^[0-9]+$/) {
	    if (($response >= $min) && ($response <= $max)) {
		$choice = $response;
		last;
	    }
	}
    }

    return ($choice);
}

# sub readline (prompt)
#
# Displays a prompt and then reads a line of characters from the STDIN, 
# which is return without the newline. The result is echoed to STDOUT.
# NOTE: This circumvents the problem with mixing readkey() and <STDIN>
#
sub readline {
    my($prompt) = @_;
    my($line) = "";
	my($char) = "";

    # Read the line a character at a time
    print $prompt;
    while ($char ne "\n") {
	$char = &readkey();
	$line .= $char;
    }
    chomp $line;
    print "$line\n";

    return ($line);
}

# normalize_numeric_array(list): convert an array of counts into relative percents
#
sub normalize_numeric_array {
    my(@vector) = @_;
    my($sum) = 0.0;    

    for (my $i = 0; $i <= $#vector; $i++) {
	$sum += $vector[$i];
    }
    if ($sum > 0) {
	for ($i = 0; $i <= $#vector; $i++) {
	    $vector[$i] = $vector[$i] / $sum;
	}
    }

    return (@vector);
}

# calc_entropy(dist_array): Calculate entropy for the distribution of values in the array
#
sub calc_entropy {
    my(@dist) = @_;
    &debug_print(&TL_VERBOSE, "calc_entropy(@_)\n");
    my($LOG2) = log(2);
    my($entropy) = 0.0;
    my($max_prob) = 0.0;
    my($p_lg_p);
    my($prob);

    foreach $prob (@dist) {
	$max_prob = &max($prob, $max_prob);
	$p_lg_p = 0;
	if ($prob > 0) {
	    $p_lg_p = - $prob * (log($prob)/$LOG2);
	}
	$entropy += $p_lg_p;
	&debug_out(&TL_VERBOSE, "p=%.3f; p lg(p)=%.3f\n", $prob, $p_lg_p);
    }
    &debug_out(&TL_VERBOSE, "entropy=%.3f\n", $entropy);

    return ($entropy);
}

#------------------------------------------------------------------------------

# return successful-load status
&debug_print(&TL_VERBOSE, "extra.perl loaded OK\n");
1;
