# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#
#!/usr/bin/perl -sw
#
# extract_speech_part_features.perl: extract features indicative of speech part usage
# from denotational assertions
# 
# note: this previously was part of make_pos_features.perl, which also formatted the
# feature tables for use in machine learning (ML)
#
# Portions Copyright (c) 2001 Cycorp, Inc.  All rights reserved.
#
# TODO:
# - have an option to filter types by generality estimate (eg, upper and lower bounds)
#

# Load in the common module, making sure the script dir is first in Perl's lib path
BEGIN { my $dir = `dirname $0`; chomp $dir; unshift(@INC, $dir); }
require 'common.perl';
require 'extra.perl';
require 'NL_support.perl';

# Process the command-line options
if (!defined($ARGV[0])) {
    my($options) = "options = [-max_instances=N]";
    my($example) = "examples:\n\n$script_name .EnglishMt.data\n\n";
    my($note) = "";

    die "\nusage: $script_name [options] mt-listing ...\n\n$options\n\n$example\n$note\n";
}

&init_var(*max_instances, &MAXINT);	# maximum number of instances or cases to output
&init_var(*direct, &FALSE);		# just use directly asserted types
&init_var(*show_headword, &FALSE);	# show the headword for the wordunit and POS
&init_var(*separate, &FALSE);		# use separate features for isa and genls
&init_var(*conflate, ! $separate);	# conflate the isa and genls types
&init_var(*count, &FALSE);		# just count the predicates

# Init TCP access to Cyc server and load in supporting lisp code if necessary
&init_NL_support() unless ($count);

# Misc. initializations
my($instance_count) = 0;

my($fn_generalizations) = ($direct ? \&direct_term_generalizations : \&term_generalizations);
my($fn_types) = ($direct ? \&direct_term_types : \&term_types);
my(%pred_count);
my($total) = 0;
my(%speech_part_count);

# Check for denotational assertions in MT listing(s) and extract the features
#
while (<>) {
    chomp;
    &dump_line();
    my($assertion) = $_;
    my($pre_head, $headword_unit, $post_head, $speech_part, $term);

    # Extract the term (i.e., denotatum) information from the semantic assertions
    #
    # ex: (multiWordString ("police") Officer-TheWord SimpleNoun LawEnforcementOfficer)
    if ($assertion =~ /\[\S+\] \(multiWordString \(([^\)]+)\) (\S+) (\S+) (.*)\)/) {
	($pre_head, $headword_unit, $speech_part, $term) = ($1, $2, $3, $4);
	$pred = "multiWordString";
    }
    # ex: (compoundString System-TheWord ("of" "belief") SimpleNoun BeliefSystem)
    elsif ($assertion =~ /\[\S+\] \(compoundString (\S+) \(([^\)]+)\) (\S+) (.*)\)/) {
	($headword_unit, $post_head, $speech_part, $term) = ($1, $2, $3, $4);
	$pred = "compoundString";
    }
    # ex: (headMedialString ("formal") Code-TheWord ("of" "conduct") SimpleNoun FormalCOC)
    elsif ($assertion =~ /\[\S+\] \(headMedialString \(([^\)]+)\) (\S+) \(([^\)]+)\) (\S+) (.*)\)/) {
	($pre_head, $headword_unit, $post_head, $speech_part, $term) = ($1, $2, $3, $4, $5);
	$pred = "headMedialString";
    }
    # ex: (denotation Symposium-TheWord SimpleNoun 0 MeetingTakingPlace)
    elsif ($assertion =~ /\[\S+\] \(denotation\S* (\S+) (\S+) \d+ (.*)\)/) {
	($headword_unit, $speech_part, $term) = ($1, $2, $3);
	# TODO: record the number of different denotation preds
	$pred = "denotation";
    }
    next if (!defined($term));

    # Update the statistics
    $total++;
    &incr_entry(\%pred_count, $pred);
    &incr_entry(\%speech_part_count, $speech_part);
    &debug_print(&TL_DETAILED, "# assertion='$assertion'\n");
    next if ($count);

    # Determine the types associated with the term
    my(@types) = &$fn_types($term);
    my(@generalizations) = &$fn_generalizations($term);

    # Determine the wordform for the headword unit
    my($headword) = ($show_headword
		     ? &first_string_of_word_unit_and_POS($headword_unit, $speech_part)
		     : "");

    # Print the extracted features data
    print "term=$term\n";
    print "POS=$speech_part\n";
    print "headword=$headword\n" if ($show_headword);
    if ($conflate) {
	my(@parents) = @types;
	push(@parents, @generalizations);
	printf "parents=\t%s\n", join("\t", @parents);
    }
    else {
	printf "types=\t%s\n", join("\t", @types);
	printf "generalizations=\t%s\n", join("\t", @generalizations);
    }
    print "\n";

    last if ($instance_count++ > $max_instances);
}

# Display preciate counts
print "Predicate counts:\n";
print &format_freq_data(\%pred_count);
print "\n";
print "total\t$total\n";
print "\n";

print "Speech part counts:\n";
print &format_freq_data(\%speech_part_count);
print "\n";
print "total\t$total\n";
print "\n";

&exit();

#------------------------------------------------------------------------

# first_string_of_word_unit_and_POS(headword_unit, speech_part)
#
# Returns the first string associated with the headword in the given part of speech.
#
#
sub first_string_of_word_unit_and_POS {
    my($headword_unit, $speech_part) = @_;
    return (&cyc_function("(first-string-of-word-unit (fort-for-string \"${headword_unit}\") (preds-of-pos (fort-for-string \"$speech_part\")))"));
};
