# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#
#!/usr/bin/perl -sw
#
# extract_matches.perl: simple script for extracting text in a file matching a particular pattern
# This is equivalent to the following bash function
#     function extract_matches () { perl -ne "while (/$1/) { printf \"%s\\n\", \$1; s/$1//; }" $2 }
# See count_it.perl for a similar script for counting the frequency of such patterns.
#
# Copyright (c) 2001 Cycorp, Inc.  All rights reserved.
#

# Load in the common module, making sure the script dir is in the Perl lib path
BEGIN { my $dir = `dirname $0`; chomp $dir; unshift(@INC, $dir); }
require 'common.perl';

# Process the command-line options
if (!defined($ARGV[0])) {
    my($options) = "options = [-replacement=perl_template] [-fields=N] [-para]";
    my($example) = "examples:\n\nls -l | $0 \"\\.([^\\.]+\$)\"\n\n";
    $example .= "echo :trace `$0 \"^\\(define(?:-p\\S+)? (\\S+)\" lexification-wizard.lisp`\n\n";
    $example .= "$0 -replacement='\$1-\$2-method' '^\\(def-instance-method \\((\\S+) (\\S+)' /home/tom/cycl/subloop-class-example.lisp\n\n";
    $example .= "$0 -para -replacement='A \$1 : \$2' 'def\\S+\\s*(\\S+)\\s*[^\\\"]*\\s*\\\"([^\\\"]+)\\\"' parse-template-utilities.lisp\n";
    ## $example .= "$0 example2\n\n";			        # TODO: revise example 2
    my($note) = "";
    ## $note .= "notes:\n\nSome usage note.\n";			# TODO: add optional note

    die "\nusage: $0 [options]\n\n$options\n\n$example\n$note\n";
}
&init_var(*replacement, "\$1");		# pattern for replacement (e.g, \1-\2-method)
&init_var(*para, &FALSE);	# paragraph input mode
&init_var(*fields, 1);			# number of output fields
&init_var(*one_per_line, &FALSE);	# only count one instance of the pattern per line

## TODO -i option
## &init_var(*i, &FALSE);		# case insensitive
my($pattern) = shift @ARGV;

# Optionally set paragraph input mode
if ($para) {
    $/ = "";
}

# Enforce a single pattern per line when beginning-of-line pattern (^) used 
if ($pattern =~ /^\^/) {
    $one_per_line = &TRUE;
}

# Put grouping parenthesis around pattern, if none present
if ($pattern !~ /\(.*\)/) {
    $pattern = "(" . $pattern . ")";
}

# For multiple-field patternsm, generate a replacement with tab separated fillers
# note: this assumes that replacement is same as "\$1" default
my($i);
for ($i = 2; $i <= $fields; $i++) {
    $replacement .= "\t\$$i";
}

# Make sure the replacement uses $1, etc. in place of \1 to avoid Perl warning
# (e.g. "\1 better written as $1 at (eval 2) line 1, <> line 17.")
$replacement =~ s/\\(\d+)/\$$1/g;
#
&debug_out(&TL_DETAILED, "pattern='%s'; replacement='%s'\n", $pattern, $replacement);

# Scan through the text (checking the pattern line by line)
while (<>) {
    &dump_line();
    chomp;

    # Print the text instances that matches the pattern, each on a separate line
    while (s/$pattern//) {
	my($matching_text) = $&;
	&debug_out(&TL_VERBOSE, "matching text: '%s'\n", $matching_text);
	eval "\$matching_text =~ s/\$pattern/$replacement/";
	printf "%s\n", $matching_text;
	last if ($one_per_line);
    }
}

&exit();
