# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#
#!/usr/bin/perl -sw
#
# format_speech_part_features.perl: format a table of features indicative of speech
# part usage, using output from extract_speech_part_features.perl
#
# Portions Copyright (c) 2001 Cycorp, Inc.  All rights reserved.
#
# Sample input format:
#
# assertion='   ist : [Inf] (denotation Amorphous-TheWord Adverb 0 Amorphous)'
# term=Amorphous POS=Adverb
# parents=	Intangible	PartiallyIntangible	PartiallyIntangibleIndividual	IntangibleIndividual	(CollectionUnionFn (TheSet AttributeValue AbstractInformationalThing))	Location-Underspecified	Trajector-Underspecified	Individual	(MeaningInSystemFn SENSUS-Information1997 "ABSTRACT-OBJECT")	AttributeValue	PhysicalAttribute	ShapeAttribute	Thing	HPKBDisplayConstant	IKBConstant	CPOFConstant	Amorphous
#
#

# Load in the common module, making sure the script dir is first in Perl's lib path
BEGIN { my $dir = `dirname $0`; chomp $dir; unshift(@INC, $dir); }
require 'common.perl';
require 'NL_support.perl';

# Process the command-line options
if (!defined($ARGV[0])) {
    my($options) = "options = [-num_type_features=N] [-type_file=file] [-specific_types]";
    my($example) = "examples:\n\n$script_name full_pos_features.log\n\n";
    my($note) = "Note: By default, 10 feature buckets are used.";

    die "\nusage: $script_name [options] mt-listing ...\n\n$options\n\n$example\n$note\n";
}

&init_var(*num_type_features, 10);	# number of hashed type feature positions
&init_var(*type_file, "");		# file with list of cyc types to check
&init_var(*generalization_file, "");	# file with list of cyc generalizations to check
&init_var(*specific_types,		# list of specific types to use
	  ($type_file ne "") ? &read_file($type_file) : "");
&init_var(*specific_generalizations,	# list of specific generalizations to use
	  ($generalization_file ne "") ? &read_file($generalization_file) : "");
&init_var(*separate, &FALSE);		# use separate features for isa and genls
&init_var(*conflate, ! $separate);	# conflate the isa and genls types
&init_var(*use_hashed_types, 		# use hashed type enumerations be used
	  ($specific_types eq ""));	# (vs. binary features for each type)
&init_var(*mapping_file, "");		# file with the type mapping
my($map_types) = ($mapping_file ne "");

# Misc. initializations
my($default_feature_value) = ($use_hashed_types ? "-" : "0");
my(@specific_types) = &tokenize($specific_types);
my(@specific_generalizations) = &tokenize($specific_generalizations);

# Initialize the optional mapping, revising the reference list accordingly
my(%type_map);
if ($map_types) {
    # Read in the mapping into the associative array
    &init_mapping(\%type_map, $mapping_file);

    # Convert the list of reference types using the mapping
    my(@new_types) = &map_all_types(\%type_map, @specific_types);
    @specific_types = &remove_duplicates(@new_types);
    my(@new_generalizations) = &map_all_types(\%type_map, @specific_generalizations);
    @specific_generalizations = &remove_duplicates(@new_generalizations);
}

# Other initializations
$num_type_features = (scalar @specific_types);
$num_generalization_features = (scalar @specific_generalizations);
my(@num_type_conflicts) = (0) x $num_type_features;
my(@num_generalization_conflicts) = (0) x $num_generalization_features;

# Set paragraph input mode
$/ = "";

# Print feature description line
print "POS";
my($i);
for ($i = 0; $i < $num_type_features; $i++) {
    my($feature) = ($use_hashed_types ? "F$i" : $specific_types[$i]);
    print "\tt$feature";
}
for ($i = 0; $i < $num_generalization_features; $i++) {
    my($feature) = ($use_hashed_types ? "F$i" : $specific_generalizations[$i]);
    print "\tg$feature";
}
print "\n";

# Check for denotational assertions in MT listing(s) and extract the features
#
while (<>) {
    chomp;
    &dump_line();

    # Extract the term and feature information from the listing
    my($term) = ($_ =~ /term=(\S+)/);
    my($speech_part) = ($_ =~ /POS=(\S+)/);
    my($parent_list) = ($_ =~ /parents=\t?([^\n]+)+/);
    my($type_list) = ($_ =~ /types=\t?([^\n]+)+/);
    my($generalization_list) = ($_ =~ /generalizations=\t?([^\n]+)+/);
    next unless (defined($term) && defined($speech_part));
    next if ($conflate && !defined($parent_list));
    next if ($separate && !defined($generalization_list));
    my(@parents);
    if ($conflate) {
	@parents = (split(/\t/, $parent_list));
    }
    else {
	@types = (split(/\t/, $type_list));
	@generalizations = (split(/\t/, $generalization_list));
    }

    # Optyionally, map the parent type ID's to their corresponding ID's
    # TODO: have someone at Cycorp review this code!!!
    if ($map_types) {
	@parents = &map_all_types(\%type_map, @parents);
	@types = &map_all_types(\%type_map, @types);
	@generalizations = &map_all_types(\%type_map, @generalizations);
    }

    my(@features);
    if ($use_hashed_types) {
	# TODO: add support for -separate
	@features = ($default_feature_value) x $num_type_features;

	# Assign each of the parents to the appropriate hash bucket.
	# That is, each feature Fi is a enumeration of the possible types hashing to bucket i
	for (my $i = 0; $i <= $#parents; $i++) {
	    my($position) = &type_hash_bucket($parents[$i]);
	    if ($features[$position] ne $default_feature_value) {
		&debug_out(&TL_VERBOSE, "conflict at bucket $position: %s replacing %s\n",
			   $parents[$i], $features[$position]);
		$num_type_conflicts[$position]++;
	    }
	    $features[$position] = $parents[$i];
	}
    }
    else {
	# Each feature Fi is binary indication of whether that specific type is in parent list
	# TODO: convert @specific_types into hash for efficient lookup's
	if ($conflate) {
	    @features = ($default_feature_value) x $num_type_features;
	    for (my $i = 0; $i <= $#parents; $i++) {
		my($position) = &find(\@specific_types, $parents[$i]);
		if ($position != -1) {
		    $features[$position] = "1";
		}
	    }
	}
	else {
	    # Produce features for type terms
	    my(@type_features) = ($default_feature_value) x $num_type_features;
	    for (my $i = 0; $i <= $#types; $i++) {
		my($position) = &find(\@specific_types, $types[$i]);
		if ($position != -1) {
		    $type_features[$position] = "1";
		}
	    }
	    push(@features, @type_features);

	    # Produce features for generalization terms
	    my(@generalization_features) = ($default_feature_value) x $num_generalization_features;
	    for (my $i = 0; $i <= $#generalizations; $i++) {
		my($position) = &find(\@specific_generalizations, $generalizations[$i]);
		if ($position != -1) {
		    $generalization_features[$position] = "1";
		}
	    }
	    push(@features, @generalization_features);
	}
    }

    # Print the feature list
    printf "%s\t%s\n", $speech_part, join("\t", @features);
}

# Display information on conflicts in feature assignments
if ($use_hashed_types) {
    &debug_out(&TL_DETAILED, "feature conflict counts:\n\t%s\n",
	       join("\t", @num_type_conflicts));
}

&exit();

#------------------------------------------------------------------------

# type_hash_bucket(term): return hash value for the type modulo number of buckets
#
sub type_hash_bucket {
    my($term) = @_;

    my($hash_bucket) = &hash_key($term, $num_type_features);
    &debug_print(&TL_VERY_DETAILED, "type_hash_bucket(@_) => $hash_bucket\n");

    return ($hash_bucket);
}

# bad_hash_key(key, num_buckets): hash function based on internal Perl hashing scheme
#
# from hv.h in Perl source distribution (version 5.6.1-2):
#
#  #define PERL_HASH(hash,str,len) \
#       STMT_START { \
#          register const char *s_PeRlHaSh = str; \
#          register I32 i_PeRlHaSh = len; \
#          register U32 hash_PeRlHaSh = 0; \
#          while (i_PeRlHaSh--) \
#              hash_PeRlHaSh = hash_PeRlHaSh * 33 + *s_PeRlHaSh++; \
#          (hash) = hash_PeRlHaSh + (hash_PeRlHaSh>>5); \
#      } STMT_END
#
# note: this produces too many conflicts perhaps due to faulty overflow handling
#  
sub bad_hash_key {
    my($key, $num_buckets) = @_;
    my($result) = 0;

    # Require that all operations (in this block) be done using integer arithmetic
    use integer;

    foreach my $char (split(//, $key)) {
	&debug_out(&TL_VERY_VERBOSE, "result=$result ord(char)=%d\n", ord($char));
	$result = (($result * 33) + ord($char));
	$result += &MAXINT if ($result < 0);
	## $result = -$result if ($result < 0);
    }
    $result += ($result / 32);
    $result += &MAXINT if ($result < 0);
    ## $result = -$result if ($result < 0);
    &debug_out(&TL_VERY_VERBOSE, "result=$result\n");

    return ($result % $num_buckets);
}

# bad_hash(key): returns hashing value given string key
#
# from Perl 5 Internals: http://www.netthink.co.uk/downloads/internals/x584.html
#
# note: this also produces too many conflicts
#
sub bad_hash {
    my $string = shift;
    my $hash = 0;

    for (map {ord $_} split //, $string) {
	&debug_out(&TL_VERY_VERBOSE, "result=$hash ord(char)=%d\n", $_);
	$hash += $_; $hash += $hash << 10; $hash ^= $hash >> 6;
    }
    $hash += $hash << 3; $hash ^= $hash >> 1;

    return ($hash + $hash << 15);
}

# djb_hash(key): return hash value for key using Bernstein's algorithm
#
# from http://www.cs.yorku.ca/~oz/hash.html:
# this algorithm (k=33) was first reported by dan bernstein many
# years ago in comp.lang.c. another version of this algorithm (now
# favored by bernstein) uses xor: hash(i) = hash(i - 1) * 33 ^ str[i];
# the magic of number 33 (why it works better than many other
# constants, prime or not) has never been adequately explained.
#
sub djb_hash {
    my($key) = @_;
    my($hash) = 5381;

    # Require that all operations (in this block) be done using integer arithmetic
    use integer;

    foreach my $char (split(//, $key)) {
	&debug_out(&TL_VERY_VERBOSE, "hash=$hash ord(char)=%d\n", ord($char));
	$hash = ($hash * 33) + ord($char);
	$hash += &MAXINT if ($hash < 0);
    }

    return ($hash);
}


# sdbm_hash(key): return hash value for key using algorithm from SDBM
#
# from http://www.cs.yorku.ca/~oz/hash.html:
# this algorithm was created for sdbm (a reimplementation of ndbm)
# database library. it does well in scrambling bits, causing better
# distribution of the keys and fewer splits. it also happens to be a
# good general hashing function with good distribution. the actual
# function is hash(i) = hash(i - 1) * 65599 + str[i]; what is included
# below is the faster version used in gawk. [there is even a faster,
# duff-device version] the magic constant 65599 was picked out of thin
# air while experimenting with different constants, and turns out to
# be a prime. this is one of the algorithms used in berkeley db (see
# sleepycat) and elsewhere.
#
sub sdbm_hash {
    my($key) = @_;
    my($hash) = 0;
    
    # Require that all operations (in this block) be done using integer arithmetic
    use integer;

    foreach my $char (split(//, $key)) {
	&debug_out(&TL_VERY_VERBOSE, "hash=$hash ord(char)=%d\n", ord($char));
	$hash = ord($char) + ($hash << 6) + ($hash << 16) - $hash;
	$hash += &MAXINT if ($hash < 0);
    }

    return ($hash);
}


sub hash {
    return (&sdbm_hash(@_));
}

# hash_key(key, num_buckets): returns hash value restricted to given number of buckets
#
sub hash_key {
    my($key, $num_buckets) = @_;
    return (&hash($key) % $num_buckets);
}

# init_mapping(hash_ref, file): Initialize the mapping of type identifiers.
# The file is a list of tab-delimited pairs giving old and new ID's.
#
sub init_mapping {
    my($hash_ref, $file) = @_;
    my($mapping);

   foreach $mapping (split/\n/, &read_file($file)) {
	my($cyc_type, $wordnet_type) = split(/\t/, $mapping);
	## $hash_ref->{$cyc_type} = $wordnet_type;
	&set_entry($hash_ref, $cyc_type, $wordnet_type);
    }
}

# map_type(hash_ref, id): returns new ID defined by the mapping
# or "" if the mapping isn't defined
#
sub map_type {
    my($hash_ref, $id) = @_;

    ## my($type) = $hash_ref->{$id};
    ## $type = "" if (!defined($type));
    my($type) = &get_entry($hash_ref, $id, "");

    return ($type);
}

# map_all_types(\%hash_ref, type, ...): returns array with list of new ID's
# for the ID's passed in, excluding cases without mappings defined
# NOTE: This basically renames the list of terms.
#
sub map_all_types {
    my($hash_ref, @old_types) = @_;
    my(@new_types);
    my($type);

    # Derive the new list of mappable types from old list
    foreach $type (@old_types) {
	my($new_type) = &map_type($hash_ref, $type);
	push(@new_types, $new_type) unless ($new_type eq "");
    }
    &debug_out(&TL_MOST_VERBOSE, "map_all_types => (@new_types)\n");

    return (@new_types);
}
