# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#
#!/usr/bin/perl -sw
#
# get_cyc_speech_part_collocations.perl: determine which Cyc types are indicative
# of each speech part. This uses deviation from conditional independence (CI) as
# measure of indicativeness.
#
# NOTE:
# This script is still work-in-progress. Currently, it just produces outputs the
# co-occurrence freuqency data for use with calc_cooccurrence.perl
#
# TODO:
# - Use the cooccurrence.perl module to derive the metrics of calc_cooccurrence.perl 
# - Complete the script: output collocation lists rather than cooccurrence data
#
# Portions Copyright (c) 2001 Cycorp, Inc.  All rights reserved.
#
# Sample input format:
#
# assertion='   ist : [Inf] (denotation Amorphous-TheWord Adverb 0 Amorphous)'
# term=Amorphous POS=Adverb
# parents:	Intangible	PartiallyIntangible	PartiallyIntangibleIndividual	IntangibleIndividual	(CollectionUnionFn (TheSet AttributeValue AbstractInformationalThing))	Location-Underspecified	Trajector-Underspecified	Individual	(MeaningInSystemFn SENSUS-Information1997 "ABSTRACT-OBJECT")	AttributeValue	PhysicalAttribute	ShapeAttribute	Thing	HPKBDisplayConstant	IKBConstant	CPOFConstant	Amorphous
#
# Current sample output:
#
# # AB    A~B     ~AB     ~A~B    Class   	  Token
# 28      18948   0       1650646 SimpleNoun      ASCIICompatibleCharacter
# 1       2084    0       1667537 Verb    	  AbandoningSomething
# 1       18975   0       1650646 SimpleNoun      Abbreviation
# 4       18972   1       1650645 SimpleNoun      AbilityLevelAttribute
# 4       2081    2       1667535 Verb    	  Ablation
# ...
# 1       848     0       1668773 ProperCountNoun YugoslavianCar
# 3       1161    0       1668458 AgentiveNoun    Zoologist#
#

# Load in the common module, making sure the script dir is first in Perl's lib path
BEGIN { my $dir = `dirname $0`; chomp $dir; unshift(@INC, $dir); }
require 'common.perl';
require 'NL_support.perl';

# Process the command-line options
if (!defined($ARGV[0])) {
    my($options) = "options = [-nats]";
    my($example) = "examples:\n\n$script_name full_pos_features.log\n\n";
    my($note) = "";

    die "\nusage: $script_name [options] mt-listing ...\n\n$options\n\n$example\n$note\n";
}

&init_var(*nats, &FALSE);		# include non-atomic terms

# Set paragraph input mode
$/ = "";

# Initialize the tables with the co-occurrence counts
my(%class_count, %class_cooccurrence);
my(%token_count);
my($num_classes) = 0;
my($num_tokens) = 0;

# Check each assertion/type-info case in output of extract_cyc_speech_part_features.perl
#
while (<>) {
    chomp;
    &dump_line();

    # Extract the term and feature information from the listing
    my($term) = ($_ =~ /term=(\S+)/);
    my($speech_part) = ($_ =~ /POS=(\S+)/);
    my($parent_list) = ($_ =~ /parents=\t?([^\n]+)+/);
    next unless (defined($term) && defined($speech_part) && defined($parent_list));
    my(@parents) = (split(/\t/, $parent_list));

    # Each feature Fi is binary indication of whether that specific type is in parent list
    &incr_entry(\%class_count, $speech_part);
    $num_classes++;
    for (my $i = 0; $i <= $#parents; $i++) {
	if (($parents[$i] =~ " ") && (! $nats)) {
	    next;
	}
	$num_tokens++;
	&add_to_occurrence_table($speech_part, $parents[$i]);
    }
}


# Display the co-occurrence info
&determine_cooccurrence_info();

# Display the collocations
&output_collocations();

&exit();

#------------------------------------------------------------------------

# add_to_occurence_table(class, token)
#
# Add an entry for the frequency table for the class-value/collocate pair.
# If there is already an entry, increment the count
#
# globals:
#    class_cooccurrence		frequency of class-value/collocate pairs
#    token_count      		frequency of particular tokens
#
sub add_to_occurrence_table {
    my($class, $token) = @_;
    my($key);
    &debug_out(6, "add_to_occurrence_table(@_)\n");

    # Make sure there is a table for the class
    my($class_table) = &get_entry(\%class_cooccurrence, $class, {});
    &set_entry(\%class_cooccurrence, $class, $class_table);

    # Increment the occurrence count for the class/word pair
    &incr_entry($class_table, $token);
    
    # Increment the occurrence count for the word itself
    &incr_entry(\%token_count, $token);

    return;
}


# determine_cooccurrence_info()
#
# Determine the conditional probability of each class given the token values.
# During debugging, displays a table giving the frequency with which the classes
# and tokens cooccur.
#
# example output:
#    # CondPr	f/N   Class   Token
#    1.00	1/1   se.ms   approach
#    1.00	1/1   se.ms   president
#    1.00	1/1   other   guide
#    1.00	2/2   other   rate
# TODO: redo the 
#
# globals:
#    class_cooccurrence		frequency of class-value/collocate pairs
#    token_count      		frequency of particular tokens
#
sub determine_cooccurrence_info {
    my($class, $token);
    &debug_out(&TL_DETAILED, "determine_cooccurrence_info(@_)\n");

    &debug_out(&TL_VERBOSE, "# CP\tf(C T)\tf(T)\tClass\tToken\n");
    foreach $class (sort(keys(%class_cooccurrence))) {
	my($class_table) = &get_entry(\%class_cooccurrence, $class);
	foreach $token (sort(keys(%token_count))) {
	    my($cooccur_count) = &get_entry($class_table, $token);
	    my($token_count) = &get_entry(\%token_count, $token, 0);
	    if ($token_count == 0) {
		&error_out("Unexpected token count for '$token'\n");
		next;
	    }

	    my($conditional_prob) = $cooccur_count / $token_count;
	    &debug_out(&TL_VERBOSE, "%.3f\t%d\t%d\t%s\t%s\n", 
		       $conditional_prob, $cooccur_count, $token_count, $class, $token);
	    # Maintain the maximal association for the token
	    if ($conditional_prob > &get_entry(\%max_conditional_prob, $token, 0)) {
		&set_entry(\%max_conditional_prob, $token, $conditional_prob);
		&set_entry(\%max_class, $token, $class);
		## &set_entry(\%max_score, $token, $conditional_prob * $token_count);
	    }
	}
    }
}


# output_collocations(): Outputs the collocations in order of  conditional probability
# and frequency of co-occurence
#
sub output_collocations {
    &debug_out(&TL_DETAILED, "output_collocations(@_)\n");
    my($class);

    foreach $class (sort(keys(%class_count))) {
	my($class_prob) = &get_entry(\%class_count, $class, 0) / $num_classes;
	&debug_out(&TL_DETAILED, "# P($class) = %.3f\n", $class_prob);
    }

    print "# AB\tA~B\t~AB\t~A~B\tClass\tToken\n";
    foreach $token (sort(keys(%max_conditional_prob))) {
	my($max_class) = &get_entry(\%max_class, $token);
	my($f_AB) = $class_cooccurrence{$max_class}{$token};
	my($f_A) = &get_entry(\%class_count, $max_class);
	my($f_B) = &get_entry(\%token_count, $token);
	my($f_A_notB) = $f_A - $f_AB;
	my($f_notA_B) = $f_B - $f_AB;
	my($f_notA_notB) = $num_classes + $num_tokens - ($f_AB + $f_A_notB + $f_notA_B);
	
	printf "%d\t%d\t%d\t%d\t%s\t%s\n",
   	       $f_AB, $f_A_notB, $f_notA_B, $f_notA_notB, $max_class, $token;
    }
}
