# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#!/usr/bin/perl -sw
#
# paste.perl: Utility similar to Unix paste command for joining columns.
# This has the additional option of combining columns based on a key value,
# which allows for combing columns with gaps.
#

# Load in the common module, making sure the script dir is in Perl's lib path
BEGIN { $dir = `dirname $0`; chop $dir; unshift(@INC, $dir); }
require 'common.perl';

&init_var(*keys, &FALSE);		# column 1 specifies keys of lines to be joined
&init_var(*filter, "");			# pattern for lines to skip
&init_var(*comments, &FALSE);		# include comments in the result
&init_var(*default, "n/a");		# default value for the rows
&init_var(*fold_case, &FALSE);		# convert keys to lowercases
&init_var(*preserve_case, ! $fold_case); # maintain the case of keys
&init_var(*labels, "");			# labels for the column headers
&init_var(*undefined, "");		# value to use for undefined data

if (!defined($ARGV[0])) {
    $options = "options = [-keys] [-labels=list] [-filter=\"pattern\"] [-default=value] [-fold_case]";
    $example = "examples:\n\n$script_name -keys eval_061798.report eval_061898.report\n\n";
    $example .= "$script_name -keys manufacturing.list construction.list | perl -pe \"s/^([^\\t]+)\\t(.*)/\\2\\t\\1/;\" >| combined.list\n";

    die "\nusage: $script_name [options]\n\n$options\n\n$example\n\n";
}

local(%all_keys);
local($num_cols) = (1 + $#ARGV);
local($max_row) = 0;
local($i);
local(@labels) = ($labels ne "") ? &tokenize($labels) : @ARGV;

# Read in all the files into separate column arrays
# For keys, these are %col_0, %col_1, ..., otherwise @col_0, @col_1, ....
#
for ($i = 0; $i < $num_cols; $i++) {
    local(*col_array) = eval "*col_$i";
    local($num) = &read_columns($ARGV[$i], *col_array);
    $max_row = &max($max_row, $num);
}

# Print the columns row by row. If by key, then the first column is the
# key and a column entry is printed if it corresponds to the key.
# 
#
if ($keys) {
    &print_columns_by_key();
}
else {
    &print_columns_by_row();
}

#------------------------------------------------------------------------------

# read_columns(file_name, column_array_ref)
#
# Read the data in the file and store into the column array. If by-key, then
# the data is assumed to be in two columns with the first providing the key
# and the second the value for the entry column_array<key>. Otherwise,
# the line is stored as colum_array[row].
#
# Returns the number of rows read.
#
sub read_columns {
    local ($file, *col_array) = @_;
    &debug_out(4, "read_columns(@_)\n");
    local($num) = 0;

    if (!open(FILE, "<$file")) {
	&exit("Unable to read file $file ($!)\n");
	return;
    }
    while (<FILE>) {
	&dump_line();
	chop;
	$num++;
	next if ((! $comments) && /^\#/);

	# Skip the line if it matches the filter
	if (($filter ne "") && ($_ !~ /$filter/)) {
	    &debug_out(5, "filtering out data: $_\n");
	    next;
	}

	# Add the data to the appropriate array
	if ($keys) {
	    ## local($key, $data) = &tokenize($_);
	    local($key, $data) = ($_ =~ /(.*)\t(.*)/);
	    $key = $_ if (!defined($key));
	    $key = &to_lower($key) unless ($preserve_case);
	    $data = $undefined if (!defined($data));
	    $all_keys{$key} = &TRUE;
	    $col_array{$key} = $data;
	}
	else {
	    local($data) = $_;
	    push(@col_array, $data);
	}
    }
    close(FILE);

    return ($num);
}


# Print the columns in order corresponding to the sorted list of keys.
# The data for the column entries is in %col_0, %col_1, ....
# globals: %all_keys, @labels
#
sub print_columns_by_key {
    &debug_out(4, "print_columns_by_key(@_)\n");
    local(@keys) = sort(keys(%all_keys));
    local($r, $c);

    
    # Print a comment giving the source of each column
    printf "# key";
    for ($c = 0; $c < $num_cols; $c++) {
	printf "\t%s", defined($labels[$c]) ? $labels[$c] : "?";
    }
    printf "\n";

    # Print the data saved for each key
    for ($r = 0; $r <= $#keys; $r++) {
	printf "%s", $keys[$r];
	for ($c = 0; $c < $num_cols; $c++) {
	    local(*col_array) = eval "*col_$c";
	    printf "\t%s", &get_entry(*col_array, $keys[$r], $default);
	}
	printf "\n";
    }	    

    return;
}


# Print the columns in row order.
# The data for the column entries is in @col_0, @col_1, ....
#
sub print_columns_by_row {
    &debug_out(4, "print_columns_by_row(@_)\n");
    local($r, $c);

    for ($r = 0; $r < $max_row; $r++) {
	for ($c = 0; $c < $num_cols; $c++) {
	    local(*col_array) = eval "*col_$c";
	    printf "\t" if ($c > 0);
	    printf "%s", $col_array[$r] unless (!defined($col_array[$r]));
	}
	printf "\n";
    }	    

    return;
}
