# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#!/usr/bin/perl -sw
#
# perlgrep.perl: quick n' dirty version of (e)grep via perl regex matching
#
# This faciliates the use of tab characters, as well the use of Perl's
# extended regular expression pattern matching (eg, \d for [0-9]).
#
# NOTES:
#
#   Use the perl -0 option to allow for paragraph matching
#       perl -Ssw -00 perlgrep.perl -d=6 -i coling comps.bib | & less
#   NOTE: The -para option now supports this
#

# Load in the common module, making sure the script dir is in the Perl lib path
BEGIN { my $dir = `dirname $0`; chomp $dir; unshift(@INC, $dir); }
require 'common.perl';

&init_var(*v, &FALSE);		# grep -v option to show lines not matching
&init_var(*i, &FALSE);		# grep -i for case insensitive matching
&init_var(*para, &FALSE);	# paragraph input mode
&init_var(*A, 0);		# grep 'after' context option
&init_var(*B, 0);		# grep 'before' context option
&init_var(*n, &FALSE);		# grep line-number option
## &init_var(*C, 0);		# grep 'context' (i.e., before & after) option
&init_var(*max, &MAXINT);	# maximum number of matches to show
&init_var(*show_filename, &FALSE);	# include file name in output
## TODO: &init_var(*c, &FALSE);		# just show count
&init_var(*w, &FALSE);		# match word boundaries

if (!defined($ARGV[0])) {
    $options = "options = [-v] [-i] [-para] [-A=n] [-max=N] [-show_filename]";
    $example = "examples:\n\n$script_name -v \"\\t\\t\" 052698_nb_ci_062498_nb55.diff\n\n";
    $example .= "perlgrep.perl -para -i \"coling\" comps.bib | & less\n\n";
    $example .= "$0 -show_filename -max=1 \"\\r\\n\" *.lisp\n\n";

    die "\nusage: $script_name [options]\n\n$options\n\n$example\n\n";
}

$pattern = shift @ARGV;

select(STDOUT); $| = 1;		    # set stdout unbuffered

# Optionally set paragraph input mode
if ($para) {
    $/ = "";
}

# Modify the pattern to reflect word boundaries
if ($w) {
    $pattern = "\\b$pattern\\b";
}

my(@before);			# lines before current target context

&debug_out(5, "checking for pattern '$pattern'\n");
my($hit_count) = 0;
$after_context = 0;
my($current_file) = defined($ARGV[0]) ? $ARGV[0] : "<stdin>";
while (<>) {
    &dump_line("$_", &TL_VERY_VERBOSE);
    $include = &FALSE;
    my($filespec) = ($show_filename ? "${current_file}: " : "");

    if (($i && /$pattern/i) || (/$pattern/)) {
	$include = (! $v);
	$after_context = ($include ? ($A + 1) : 0);
    }
    else {
	$include = $v;
	$after_context--;
    }
    &debug_out(7, "include=%d; after_context=%d\n", $include, $after_context);

    if ($include) {
	if ($B > 0) {
	    printf "%s", join("", @before);
	    @before = ();
	}

	if ($show_filename) {
	    print "$current_file: ";
	}
	if ($n) {
	    printf "%d: ", $.;
	}
	printf "%s", $_;

	# See whether to stop the search due to max hit count
	if (++$hit_count >= $max) {
	    &debug_out(4, "max hits (%d) reached\n", $max);
	    last;
	}
    }
    elsif ($after_context > 0) {
	printf "%s", $_;
    }
    elsif ($B > 0) {
	# update the lines-before context
	# NOTE: This is only done for lines not otherwise printed
	# TODO: look into a more efficient queue implementation
	push (@before, $_);
	shift @before if ($#before = $B);
    }
    if (eof) {
	$current_file = defined($ARGV[0]) ? $ARGV[0] : "<stdin>";
    }
}
