#! /bin/csh -f
#
# preprocess-lexical-data.sh: Extracts lexical data from the Cyc English
# lexicon and processes it into a format for use in the speech part 
# classification experiments.
#

# Uncomment (or comment) the following for enabling (or disabling) tracing
## set echo=1

# Parse command-line arguments
if ("" == "`printenv port`") setenv port 40
set force=1
if ("$1" == "") then
    echo ""
    echo "usage: `basename $0` [--port N]"
    echo ""
    echo "ex: `basename $0` --port 20 > preprocess-lexical-data.log"
    echo ""
    exit
endif
while ("$1" =~ -*)
    if ("$1" == "--port") then
	setenv port $2
	shift
    else if ("$1" == "--quick") then
	set force = 0
    else
	echo "unknown option: $1"
    endif
    shift
end

# Display information about the version of Cyc being used
@ tcp_port = $port + 3602
echo "Using Cyc at port offset $port; TCP port is $tcp_port"

# Download the MT's
echo "Extracting lexical MT data"
foreach mt (GeneralLexiconMt GeneralEnglishMt EnglishMt WordNetMappingMt) 
    if ($force || (-z .${mt}.data.gz)) then
        echo "Extracting $mt"
        lynx -dump -nolist -width=512 "http://localhost:${tcp_port}/cgi-bin/cyccgi/cg?cb-c-total&"${mt} > .${mt}.data
        lynx -dump -nolist -width=512 "http://localhost:${tcp_port}/cgi-bin/cyccgi/cg?cb-c-ist&"${mt} >>.${mt}.data
        gzip -f .${mt}.data
    endif
end

# Download other Cyc data (e.g., quotedCollection's)
perl -Ssw query_cyc.perl '(quotedCollection ?C)' > quotedCollection.list

# Extract the denotational assertions (denotation and multi-word assertions)
zcat .EnglishMt.data.gz .GeneralEnglishMt.data.gz .GeneralLexiconMt.data.gz |  perl -Ssw extract_cyc_speech_part_features.perl -separate -d=4 -port=$port - >& extract_cyc_speech_part_full.log
gzip -f extract_cyc_speech_part_full.log
