# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#
#!/usr/bin/perl -sw
#
# produce_cyc_wordnet_mapping.perl: given a list of Cyc terms this produces a table showing the
# corresponding WordNet synset
#
# Portions Copyright (c) 2001 Cycorp, Inc.  All rights reserved.
#

# Load in the common module, making sure the script dir is first in Perl's lib path
BEGIN { my $dir = `dirname $0`; chomp $dir; unshift(@INC, $dir); }
require 'common.perl';

require 'NL_support.perl';

# Process the command-line options
if (!defined($ARGV[0])) {
    my($options) = "options = [n/a]";
    my($example) = "examples:\n\n$script_name cyc_parent_terms.list\n\n";
    my($note) = "";

    die "\nusage: $script_name [options] mt-listing ...\n\n$options\n\n$example\n$note\n";
}

# Init TCP access to Cyc server and load in supporting lisp code if necessary
&init_NL_support();

# Get synset for each term in the input (one per line)
while (<>) {
    chomp; s/\r//;
    &dump_line();
    my($cyc_term) = $_;

    # Show the corresponding synset for the term
    my($synset) = &determine_wordnet_mapping($cyc_term);
    print "$cyc_term\t$synset\n";

}

&exit();

#------------------------------------------------------------------------

# determine_wordnet_mapping(cyc_term, [depth]): returns the ID of the WordNet
# synset corresponding to term or to its most specific generalization
# NOTE: This is a recursive function that uses memoization to avoid
# duplucate work.
# ex: determine_wordnet_mapping(Dog) => "N01595188"
# ex: determine_wordnet_mapping(AustralianShepherd) => "N01595188"
#
my(%linked_synset);
#
sub determine_wordnet_mapping {
    my($cyc_term, $depth) = @_;
    $depth = 0 if (!defined($depth));
    my($indent) = " " x $depth;
    my($synset);

    # First check the cache of linked synsets
    if (defined($linked_synset{$cyc_term})) {
	return ($linked_synset{$cyc_term});
    }

    # Check for a direct link
    $synset = get_linked_synset($cyc_term);

    # If not linked, check each of the parents in turn
    # TODO: sort the parents by generality before recursing
    if ($synset eq "") {
	foreach my $parent (&direct_term_generalizations($cyc_term)) {
	    $linked_synset{$cyc_term} = "";	# to avoid infinite recursion
	    $synset = &determine_wordnet_mapping($parent, $depth + 1);
	    last if ($synset ne "");
	}
    }
    $synset = "???" if ($synset eq "");

    # Save the results in the cache
    &debug_print(&TL_DETAILED, "${indent}determine_wordnet_mapping(@_) => '$synset'\n");
    $linked_synset{$cyc_term} = $synset;

    return ($synset);
    
}

# get_linked_synset(cyc_term): return ID of WordNet synset corresponding to the term
# based on explcit synonymousExternalConcept mappings
# ex: get_linked_synset(Beagle) => "N01599092"
#
# Mt : WordNetMappingMt
#  (synonymousExternalConcept Beagle WordNet-1997Version "N01599092")
#
sub get_linked_synset {
    my($cyc_term) = @_;
    ## my($version) = "WordNet-1997Version";

    my($synset) = &cyc_string_function_fmt("(fpred-value-in-mt (fort-for-string \"%s\") \#\$synonymousExternalConcept \#\$WordNetMappingMt 1 3)", $cyc_term);
    &debug_print(&TL_DETAILED, "get_linked_synset(@_) => '$synset'\n");

    return ($synset);
}
