# *-*-perl-*-*
eval 'exec perl -Ssw $0 "$@"'
    if 0;
#
#!/usr/bin/perl -sw
#
# Test of using Perl Cyc module (by Charles Sutton). This takes a query specified
# on the command line and invokes the Cyc module to perform the query, by default
# using InferencePSC.
#
# examples:
#
#    bash$ query_cyc.perl '(verbSemTrans Sleep-TheWord 0 ?Frame ?Template)'
#    IntransitiveFrame	(and (isa :ACTION Sleeping) (bodilyDoer :ACTION :SUBJECT))
#
#    bash$ query_cyc.perl '(denotation Sleep-TheWord ?Sense ?POS ?Term)'
#    MassNoun	0	Sleeping
#    Verb	0	Sleeping
#    SimpleNoun	0	Sleeping
#    Adjective	0	Sleepy
#
# TODO: print the variable names for the bindings at the top of the listing
#
# Copyright (c) 2000 - 2001 Cycorp, Inc.  All rights reserved.
#

# Load in the common module, making sure the script dir is in Perl's lib path
$dir = `dirname $0`; chop $dir; unshift(@INC, $dir);
require 'common.perl';		# Tom's common module (see /home/tom/BIN)
use Cyc;			# Charles's Perl module
require 'cyc_helper.perl';	# Tom's supplementary module
use vars qw /$port $host/;

# Initialize options
#  &init_var(*host, "localhost");	# host name of the Cyc server
#  &init_var(*port, 3601);		# use 3621 for localhost20, 3641 for localhost40, etc.
&init_var(*mt, "InferencePSC");	# MT to use for the query
#  &init_var(*debug, &FALSE);	# debug mode for the Cyc module
&init_var(*verbose, &FALSE);	# verbose display of results (full query & option values)
&init_var(*nonask, &FALSE);	# assertion is not an ask
&init_var(*raw, $nonask);	# raw display of the results (text from TCP server)
&init_var(*backchain, &FALSE);	# should backchaining be used (or max number of steps)
&init_var(*number, 100000);	# max bindings returned
&init_var(*time, 60);		# max time for query
&init_var(*depth, 30);		# depth cutoff
&init_var(*cycl, $nonask);	# already in proper Cycl
&init_var(*keep_cycl, &FALSE);	# output the result in proper CycL (eg, using #$'s)
$cyclify = (! $cycl);		# should the input be cyclified
&init_var(*nowrap, &FALSE);	# disable assertion wrapping
&init_var(*wrap, &FALSE);	# enable assertion wrapping
&init_var(*disable_cutoff, &FALSE); # disable the inference cutoff

# Show the usage statement if no arguments specified
if (!defined($ARGV[0])) {
    $options =  "options = [-mt=name] [-port=num] [-debug] [-verbose] [-cycl]\n";
    $options .= "          [-backchain=N] [-number=N] [-time=N] [-depth=N] [-disable_cutoff]\n";
    $example = "examples:\n\n$script_name '(\#\$and (\#\$genls \#\$Custard ?X) (\#\$genls \#\$ChiliConCarne  \?X))'\n\n";
    $example .= "$script_name -verbose -mt=CyclistsMt '(\#\$isa \#\$OHara \?X)'\n\n";
    $example .= "$script_name -wrap -debug -\n\n";

    die "\nusage: $script_name [options]\n\n$options\n\n$example\n\n";
}

# Initialize the TCP Cyc interface
&init_cyc();

# See if particular Cyc options are being set
# TODO: get these to work properly
if ($nowrap) {
    $cyc->fi_send( "(fi-get-parameter '*cb-c-wrap-assertions*)" );
    $cyc->fi_send( "(fi-set-parameter '*cb-c-wrap-assertions* NIL)" );
}
if ($wrap) {
    $cyc->fi_send( "(fi-get-parameter '*cb-c-wrap-assertions*)" );
    $cyc->fi_send( "(fi-set-parameter '*cb-c-wrap-assertions* T)" );
}

# Convert the commandline into a Cyc query. By default, it will be Cyclified
$command_line = join(" ", @ARGV);
if (($command_line eq "-") || ($command_line eq "")) {
    &exit();
}$command_line = "(" . $command_line . ")" if ($command_line !~ /^\(/);
if ($cyclify) {
    $command_line =~ s/([ \(])([a-z-]+)/$1\#\$$2/ig;
}

# Optionally, Temporarily disable the infernce cutoff
my($old_cutoff) = 10000;
if ($disable_cutoff) {
    my($old_cutoff) = &cyc_function("(progn *removal-cost-cutoff*)");
    &cyc_function("(csetq *removal-cost-cutoff* 100000)");
}

# Make the query, and display the results
#
# fi-ask-int : formula mt &optional backchain? number time depth -> [ask-result]
#
# NOTE: fi-ask-int is used instead of fi-ask to avoid getting operations added to transcript
#
my($response) = "";
if ($nonask) {
    $response = &cyc_function($command_line);
    $command = $command_line;
}
else {
    $command = sprintf "\'%s \#\$$mt", join(" ", $command_line);
    my($ask_arguments) = sprintf "%s %s %s %s %s", $command, $backchain, $number, $time, $depth;
    &debug_out(5, "Issuing cyc->fi_ask(%s)\n", $ask_arguments);
    $response = $cyc->fi_ask($ask_arguments);
}
if ($verbose) {
    printf "Query: \n%s\n", $command;
    printf "Parameters:\nBackchain=%s number=%d time=%d depth=%d\n", 
           $backchain, $number, $time, $depth;
    printf "\n";
    printf "Response:\n";
}

# Reset the inference cutoff to its old value
if ($disable_cutoff) {
    &cyc_function("(csetq *removal-cost-cutoff* ${old_cutoff})");
}

# Display the results, either in raw format (god help you) or parsed.
#
if ($raw) {
    printf "%s\n", $response;
}
else {
    &parse_results($response);
}

&exit();

#------------------------------------------------------------------------

#
# parse_results($response): Parse the ask results, and display it
# in columnar format.
# TODO: print the variable names for the bindings at the top of the listing
#
# NOTE: This performs a convoluted conversion due to the way the code
# evolved over time. Specifically, the lisp string list is converted
# into Perl arrays with sublists represented by array references.
# Then the arrays for each binding are converted into either a string
# value or back into a lisp-like string list.
#
sub parse_results {
    my($response) = @_;
    my($binding_set, $binding_pair);
    &debug_out(6, "parse_results(@_)\n");

    # Handle special case of no response
    if ($response eq "NIL") {
	print "n/a\n";
	return;
    }

    # Pretty-print the binding list
    foreach $binding_set (@{scalar $cyc->lisp_read ($response)}) {
 	&debug_out(5, "binding_set=%s\n", $binding_set);
	my($indent) = "";
	foreach $binding_pair (@$binding_set) {
	    &debug_out(5, "\nbinding_pair=(%s . %s)\n", 
		       $binding_pair->[0], $binding_pair->[1]);
	    my(@binding) = @$binding_pair;
	    &trace_array(\@binding);

	    # Convert the value to a string, turning NATs into string lists
	    my($value) = $binding_pair->[1];
	    $value = "" if (!defined($value));
	    if (($#binding > 1) || ($value =~ /ARRAY\(0x/)) {
		shift @binding;		# remove variable name from list
		$value = &lispify(@binding);
	    }

	    # Drop the CycL constant prefixes
	    $value =~ s/\#\$//g unless ($keep_cycl);
						  
	    printf "%s%s", $indent, $value;
	    $indent = "\t";
	}
	print "\n";
    }
}

